/*******************************************************************************
 * Copyright (c) 2011, 2012 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: 
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.hyades.trace.ui.internal.export;

import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.hyades.models.hierarchy.AbstractTRCProcess;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCConfiguration;
import org.eclipse.hyades.models.hierarchy.TRCFilter;
import org.eclipse.hyades.models.hierarchy.TRCOption;
import org.eclipse.hyades.models.hierarchy.TRCTIProfilerCollectionMode;
import org.eclipse.hyades.models.trace.TRCAggregatedMethodInvocation;
import org.eclipse.hyades.models.trace.TRCClass;
import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCFullTraceObject;
import org.eclipse.hyades.models.trace.TRCGCEvent;
import org.eclipse.hyades.models.trace.TRCMethod;
import org.eclipse.hyades.models.trace.TRCMethodInvocation;
import org.eclipse.hyades.models.trace.TRCObject;
import org.eclipse.hyades.models.trace.TRCPackage;
import org.eclipse.hyades.models.trace.TRCProcess;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.models.trace.TRCThreadDeadEvent;
import org.eclipse.hyades.models.trace.TRCThreadEvent;
import org.eclipse.hyades.models.trace.TRCThreadRunningEvent;
import org.eclipse.hyades.trace.ui.internal.wizard.TraceWizardMessages;

/** 
 * This class uses the concept of "trace order", which is the chronological ordering of the profiling data as it appears in the original
 * profiling trace data file. That is, the order in which it was received by the workbench. We use this to
 * ensure that the order of events in the file that is exported is the same as the order of events
 * in the original file, in order to ensure that later calculations on this data are correct. 
 * */
public class TraceOutputUtil {
	
	private static void outputMethodStatistics(TRCProcess p, TRCAgentProxy ap, OutputStream os, IProgressMonitor pm, boolean outputBinaryTrace) throws IOException {
		
		pm.beginTask(TraceWizardMessages.EXPORT_WIZARD_EXP_EXEC_TO_FILE, p.getInvocations().size());
		
		// Buffer is set to 256k, diminishing returns above this number 
		os = new BufferedOutputStream(os, 256*1024);
		
		ITraceElementOutput xe;
		if(outputBinaryTrace) {
			xe = BinaryElementOutput.getInstance();
		} else {
			xe = XMLElementOutput.getInstance();
		}
		
		generateHeader(xe, os, p, ap);
		generateDefines(xe, os, p, true);

		generateMethodInvoc(xe, os, p, pm);
		
		if(!pm.isCanceled()) {
		
			generateFooter(xe, os, p, ap);
			
			pm.done();
		}
		
		os.flush();
	}
	
	/** Output the object allocs and deallocs in order of the provided maps (which are sorted by time) */
	private static void outputHeapStatsInOrder(ITraceElementOutput xe, OutputStream os, TreeMap<Double, TRCFullTraceObject> allocMap, TreeMap<Double, TRCFullTraceObject> deallocMap, IProgressMonitor pm) throws IOException {
		Iterator<Entry<Double, TRCFullTraceObject>> allocSetIt = allocMap.entrySet().iterator();
		Iterator<Entry<Double, TRCFullTraceObject>> deallocSetIt = deallocMap.entrySet().iterator();
		
		Entry<Double, TRCFullTraceObject> nextAllocEntry = null;
		Entry<Double, TRCFullTraceObject> nextDeallocEntry = null;

		long startTime = System.currentTimeMillis();
		
		boolean continueLoop = true;
		long count = 0;
		while(continueLoop) {
			// Iterate through the maps, outputing their values in order of time (the iterator handles the ordering)
			// - We look at the next alloc and next dealloc events, look at which is sooner, print that, then get the next event of that type
			
			if(nextAllocEntry == null && allocSetIt.hasNext()) nextAllocEntry = allocSetIt.next();
			if(nextDeallocEntry == null && deallocSetIt.hasNext()) nextDeallocEntry = deallocSetIt.next();
			
			if(nextAllocEntry == null && nextDeallocEntry == null) {
				continueLoop = false;
			}
			
			if(nextAllocEntry != null && nextDeallocEntry != null) {
				
				if(nextAllocEntry.getKey() <=  nextDeallocEntry.getKey()) {
					xe.output(os, nextAllocEntry.getValue(), true);
					nextAllocEntry = null;
					count++;
				} else {
					xe.output(os, nextDeallocEntry.getValue(), false);
					nextDeallocEntry = null;
					count++;
				}
				
				
			} else {
				
				if(nextAllocEntry != null && nextDeallocEntry == null) {
					xe.output(os, nextAllocEntry.getValue(), true);
					nextAllocEntry = null;
					count++;
					
				}
				
				if(nextDeallocEntry != null && nextAllocEntry == null) {
					xe.output(os, nextDeallocEntry.getValue(), false);
					nextDeallocEntry = null;
					count++;
				}
				
			}
			
			// Report work every 1000 outputs 
			if(count > 1000) {
				if(pm.isCanceled()) {
					return;
				}
				
				pm.worked((int)count);
				count = 0;				
			}
			
		}
	}
	
	@SuppressWarnings("unchecked")
	private static void outputHeapStatistics(TRCProcess p, TRCAgentProxy ap, OutputStream os, IProgressMonitor pm, boolean outputBinaryTrace) throws IOException {
		
		// Buffer is set to 256k, diminishing returns above this number 
		os = new BufferedOutputStream(os, 256*1024);

		ITraceElementOutput xe;
		if(outputBinaryTrace) {
			xe = BinaryElementOutput.getInstance();
		} else {
			xe = XMLElementOutput.getInstance();
		}
		
		generateHeader(xe, os, p, ap);
		generateDefines(xe, os, p, true);
		
		List<TRCThreadEvent> le = getSortedThreadEvents(p)._objects;
		for(TRCThreadEvent e : le) {
			outputObject(xe, os, null, e);
		}
		
		// Map of the times of all the allocation events
		TreeMap<Double, TRCFullTraceObject> allocMap = new TreeMap<Double, TRCFullTraceObject>();
		
		// Map of all the times of all the deallocation events
		TreeMap<Double, TRCFullTraceObject> deallocMap = new TreeMap<Double, TRCFullTraceObject>();
		
		long totalCount = 0;
		
		// Count total, for the job (We do this twice, here and below, in order to get the value to the job as soon as possible)
		for(TRCPackage pack : p.getPackages()) {
			for(TRCClass c : pack.getClasses()) {
				for(TRCObject o : c.getObjects()) {
					if(o instanceof TRCFullTraceObject) {
						TRCFullTraceObject e = (TRCFullTraceObject)o;
						totalCount++;						
						if(e.getCollectTime() > 0) {
							totalCount++;
						}
						
					}
				}
			}
		}
		
		pm.beginTask(TraceWizardMessages.EXPORT_WIZARD_EXP_HEAP_TO_FILE, (int)totalCount);
		
		
		// Iterate through all the objects and add them to the maps
		for(TRCPackage pack : p.getPackages()) {
			for(TRCClass c : pack.getClasses()) {
				for(TRCObject o : c.getObjects()) {
					if(o instanceof TRCFullTraceObject) {
						
						TRCFullTraceObject e = (TRCFullTraceObject)o;
						allocMap.put(e.getCreateTime(), e);
//						totalCount++;
						
						if(e.getCollectTime() > 0) {
							deallocMap.put(e.getCollectTime(), e);
//							totalCount++;
						}
						
					}
				}
			}
		}
		
		// ouput all the alloc and dealloc events
		outputHeapStatsInOrder(xe, os, allocMap, deallocMap, pm );
		
		if(pm.isCanceled()) {
			os.flush();
			return;
		}
		
		for(TRCGCEvent e : p.getGcEvents()) {
			xe.output(os, e);
		}

		generateFooter(xe, os, p, ap);
		
		os.flush(); // flush our buffered os
		
		pm.done();
		
	}
	
	
	/** Utility method that determines the profiling mode from the profiling options */
	public static TRCTIProfilerCollectionMode calcTICollectionMode(TRCAgentProxy ap, TRCAgent aa) {
		for(TRCConfiguration t : ap.getConfigurations()) {
			if(t.getName().equalsIgnoreCase("__JVMPI__")) { // Actually JVMTI //$NON-NLS-1$
				boolean isHeap = false;
				boolean isHeapObjAllocSites = false;
				
				boolean isExec = false;
				boolean isExecFlow = false;
				
				boolean isThread = false;
//				boolean isThreadCont = false;
				
				
				for(TRCOption o : t.getOptions()) {
					if(o.getKey().equalsIgnoreCase("org.eclipse.tptp.platform.jvmti.client.ATTR_HEAP_COLLECT_INSTANCE")) { //$NON-NLS-1$
						isHeap = true;
						
					}
					if(o.getKey().equalsIgnoreCase("org.eclipse.tptp.platform.jvmti.client.ATTR_EXEC_OBJ_ALLOC_SITE")) { //$NON-NLS-1$
						isHeapObjAllocSites = (o.getValue() != null ? o.getValue().trim().equalsIgnoreCase("true") : false); //$NON-NLS-1$
					}
					
					if(o.getKey().equalsIgnoreCase("org.eclipse.tptp.platform.jvmti.client.ATTR_EXEC_DATA")) { //$NON-NLS-1$
						isExec = true;
					}
					
					if(o.getKey().equalsIgnoreCase("org.eclipse.tptp.platform.jvmti.client.ATTR_EXEC_FLOW")) { //$NON-NLS-1$
						isExecFlow = (o.getValue() != null ? o.getValue().trim().equalsIgnoreCase("true") : false); //$NON-NLS-1$
					}
					
				}
				
				if(isExec) {
					if(isExecFlow) {
						return TRCTIProfilerCollectionMode.EXEC_FLOW;
					} else {
						return TRCTIProfilerCollectionMode.EXEC_STATS;
					}
				} else if(isHeap) {
					if(isHeapObjAllocSites) {
						return TRCTIProfilerCollectionMode.HEAP_ALLOC_SITES;
					} else {
						return TRCTIProfilerCollectionMode.HEAP_NO_ALLOC_SITES;
					}
				} else if(isThread) {
					return TRCTIProfilerCollectionMode.THREAD_CONTENTION;
				}
				
				break;
			}
		}
		return null;
	}
	
	public static void exportAgentDataToStream(TRCAgent aa, OutputStream os, IProgressMonitor pm, boolean binaryOutputFormat) throws IOException {
		AbstractTRCProcess atp = aa.getProcess();
		TRCAgentProxy ap = aa.getAgentProxy();
		
		if(atp instanceof TRCProcess) {
			TRCProcess p = (TRCProcess)atp;
			
			TRCTIProfilerCollectionMode mode = ap.getTiCollectionMode();
			if(mode == null || mode == TRCTIProfilerCollectionMode.UNKNOWN)  {
				mode = calcTICollectionMode(ap, aa);
			}
			
			if(mode == TRCTIProfilerCollectionMode.HEAP_ALLOC_SITES || mode == TRCTIProfilerCollectionMode.HEAP_NO_ALLOC_SITES) {
				outputHeapStatistics(p, ap, os, pm, binaryOutputFormat);
			} else if(mode == TRCTIProfilerCollectionMode.EXEC_FLOW || mode == TRCTIProfilerCollectionMode.EXEC_STATS) {
				outputMethodStatistics(p, ap, os, pm, binaryOutputFormat);
			}
			
		}
	}
	
	private static void generateFooter(ITraceElementOutput xo, OutputStream os, TRCProcess process, TRCAgentProxy ap) throws IOException{
		BigDecimal s = process.getAbsoluteTime();
		xo.outputTraceEndTags(process, s, os);

	}
	
	private static void generateHeader(ITraceElementOutput xe, OutputStream out, TRCProcess process, TRCAgentProxy ap) throws IOException {
		EList<TRCConfiguration> confs = ap.getConfigurations();
		
		xe.outputTraceStartTags(out, process, ap);
		
		for(TRCConfiguration t : confs) {
			if(t.getName().equalsIgnoreCase("__JVMPI__")) { // This is actually JVMTI //$NON-NLS-1$
				
				EList<TRCFilter> filters = t.getFilters();
				for(TRCFilter f : filters) {
					xe.output(out, f);
				}
				
				EList<TRCOption> options  = t.getOptions();
				for(TRCOption o : options) {
					xe.output(out, o);
				}
				
			}
		}
		
		xe.outputRuntimeInit(out, ap.getProcessProxy().getNode(), process);
				
	}
		
	private static void generateDefines(ITraceElementOutput xe, OutputStream os, TRCProcess process, boolean outputMethods) throws IOException {
		
		EList<TRCPackage> packs = (EList<TRCPackage>)process.getPackages();
		for(TRCPackage pk : packs) {
			
			EList<TRCClass> classes = pk.getClasses();
			for(TRCClass c : classes) {

				xe.outputClassDef(c, process, os);
				
				if(outputMethods) {
					EList<TRCMethod> methods = c.getMethods();
					for(TRCMethod m : methods) {
						xe.output(os, m);
					}
				}
			}
		}
		
	}

	/**
	 * Print all the thread events in the list that occur BEFORE the given trace order; this trace order
	 * corresponds to the "time" of the next method invocation. 
	 * @param threadList List of thread events
	 * @param traceOrder The trace order of the next method invocation
	 * @throws IOException
	 */
	@SuppressWarnings("unchecked")
	private static void outputThreadEventIfNext(ITraceElementOutput xe, OutputStream os, TraceOrderList threadList, long traceOrder) throws IOException {
		
		// If the next thread events should be printed BEFORE the next method invoc event
		while(traceOrder > threadList._nextId && threadList._listSize > 0) {
			
			if(threadList._nextId != Long.MAX_VALUE) {
				
				List<TRCThreadEvent> list =  (List<TRCThreadEvent>)threadList._objects;
				
				if(list != null) {
					TRCThreadEvent te = list.get(threadList._posInList);
					outputObject(xe, os, threadList, te);

					threadList._listSize--;
					threadList._posInList++;
					threadList._nextId = getNextLowestFromList(threadList);
					
				}				
			}
		}
	}
	
	/** Recurses through the structure of a given method invocation, processing it, then calling this method on its children */
	private static void recurseMethodInvoc(ITraceElementOutput xe, OutputStream os, TRCMethodInvocation mi, MethodRecurseState state, IProgressMonitor pm) throws IOException {

		if(pm.isCanceled()) return;
		
		TraceOrderList threadList = state._threadList;

		outputThreadEventIfNext(xe, os, threadList, mi.getMethodEnterTraceOrder());
		
		// Report worked every 1000 method invocs
		state._methodCount++;
		if(state._methodCount > 1000) {
			pm.worked((int)state._methodCount);
			state._methodCount = 0;
		}
		
		if(mi.getMethodEnterTraceOrder() != -1) {
		
			// Output method enter
			if(mi instanceof TRCAggregatedMethodInvocation) {
				xe.output(os, (TRCAggregatedMethodInvocation)mi, true);
			} else {
				xe.output(os, (TRCFullMethodInvocation)mi, state._process.getAgent().getStartTime(), new BigDecimal(state._process.getAgent().getStartTime()), true);
			}
		}
		
		List<TRCMethodInvocation> invocs = mi.getInvokes();
		int listSize = invocs.size();
		for(int x = 0; x < listSize; x++) {
			TRCMethodInvocation t = (TRCMethodInvocation)invocs.get(x); 
			if(t instanceof TRCMethodInvocation && (state._mapMethodOut.get(t) == null)) {
				TRCMethodInvocation a = (TRCMethodInvocation)t;
				state._mapMethodOut.put(a, new Boolean(true));
				
				recurseMethodInvoc(xe, os, a, state, pm);
				
			}
		}
		
		outputThreadEventIfNext(xe, os, threadList, mi.getMethodExitTraceOrder());
		
		if(mi.getMethodExitTraceOrder() != -1) {
		
			// Output method exit
			if(mi instanceof TRCAggregatedMethodInvocation) {
				xe.output(os, (TRCAggregatedMethodInvocation)mi, false);
			} else {
				xe.output(os, (TRCFullMethodInvocation)mi, state._process.getAgent().getStartTime(), new BigDecimal(state._process.getAgent().getStartTime()), false);
			}
		}
		
	}

	private static class MethodRecurseState {
		Map<TRCMethodInvocation, Boolean> _mapMethodOut = new HashMap<TRCMethodInvocation, Boolean>();
		
		TraceOrderList _threadList = null;
		
		TRCProcess _process = null;
		
		long _methodCount = 0;
		
	}
	
	/** Convert all the thread events of the process into a list sorted by the order in which they occur in the original trace that we are exporting from (e.g. the trace order) */
	private static TraceOrderList getSortedThreadEvents(TRCProcess p) {
		
		TraceOrderList l = null;
		
		// Scan through all of the thread events for all threads, and combine all thread start 
		// and thread stop events to a single list of thread events. 
		if(p.getThreads() != null) {
			
			List<TRCThreadEvent> le;
			
			le = new ArrayList<TRCThreadEvent>();
			
			EList<TRCThread> threadList = p.getThreads();
			for(TRCThread t : threadList) {
				EList<TRCThreadEvent > threadEvList = t.getThreadEvents();

				if(threadEvList != null && threadEvList.size() > 0) {

					int c = 0;
					for(TRCThreadEvent te : threadEvList) {
						
						// Only the first TRCThreadRunningEvent should count as a thread start
						if(c == 0 && te instanceof TRCThreadRunningEvent) {
							le.add(te);
						}
						
						if(te instanceof TRCThreadDeadEvent) {
							le.add(te);
						}
					}
					
				}
			}
			
			// Sort the list of thread events by their trace order
			Collections.sort(le, new Comparator<TRCThreadEvent>() {

				public int compare(TRCThreadEvent o1, TRCThreadEvent o2) {
					return (int)(o1.getTraceOrder() - o2.getTraceOrder());
				}
				
			});
			
			l = new TraceOrderList();
			l._posInList = 0;
			l._listSize = le.size();
			l._objects = le;
			l._nextId = getNextLowestFromList(l);
		}
		
		return l;

	}
	
	private static void generateMethodInvoc(ITraceElementOutput xe, OutputStream os, TRCProcess process, IProgressMonitor pm) throws IOException {
		List<TRCMethodInvocation> invocs = process.getInvocations();
		
		MethodRecurseState state = new MethodRecurseState();
		
		state._process = process;
		state._threadList = getSortedThreadEvents(process);
		
		int listSize = invocs.size();
		for(int x = 0; x < listSize; x++) {
			TRCMethodInvocation t = invocs.get(x);

			if(t instanceof TRCMethodInvocation && (state._mapMethodOut.get(t) == null)) {
				
				TRCMethodInvocation a = (TRCMethodInvocation)t;
				
				state._mapMethodOut.put(a, new Boolean(true));
				
				recurseMethodInvoc(xe, os, a, state, pm);
			}
			
		}
		if(pm.isCanceled()) return;
		
		// Output any remaining thread events
		outputThreadEventIfNext(xe, os, state._threadList, Long.MAX_VALUE);
		
	}

	/** The TraceOrderList param is only used for aggMethInvoc*/
	private static void outputObject(ITraceElementOutput xe, OutputStream os, TraceOrderList l, Object o) throws IOException {
		
		if(o instanceof TRCThreadRunningEvent) {
			TRCThreadRunningEvent e = (TRCThreadRunningEvent)o;
			xe.output(os, e);
		}
				
		if(o instanceof TRCThreadDeadEvent) {
			TRCThreadDeadEvent e = (TRCThreadDeadEvent)o;
			xe.output(os, e);
		}
		
		if(o instanceof TRCGCEvent) {
			TRCGCEvent e = (TRCGCEvent)o;
			xe.output(os, e);
		}
		
		if(o instanceof TRCAggregatedMethodInvocation) {
			TRCAggregatedMethodInvocation e = (TRCAggregatedMethodInvocation)o;
			
			xe.output(os, e, l._isEnter);
			
		}
	}
	
	private static long getNextLowestFromList (TraceOrderList list) {
		
		if(list._listSize > 0) {
			Object o = list._objects.get(list._posInList);
			if(o instanceof TRCThreadRunningEvent) {
				TRCThreadRunningEvent e = (TRCThreadRunningEvent)o;
				return ((TRCThreadRunningEvent) o).getTraceOrder();
			}

			if(o instanceof TRCThreadDeadEvent) {
				TRCThreadDeadEvent e = (TRCThreadDeadEvent)o;
				return ((TRCThreadDeadEvent) o).getTraceOrder();
			}
			
			if(o instanceof TRCAggregatedMethodInvocation) {
				TRCAggregatedMethodInvocation e = (TRCAggregatedMethodInvocation)o;
				
				if(list._isEnter) {
					return e.getMethodEnterTraceOrder();
				} else {
					return e.getMethodExitTraceOrder();
				}
				
			}
			
		} 
		
		return Long.MAX_VALUE;
	}
	
	@SuppressWarnings("unused")
	@Deprecated
	private List<TraceOrderList> setupItems(TRCProcess p) {
		List<TraceOrderList> lists = new ArrayList<TraceOrderList>();
		
		EList<TRCMethodInvocation> mil = p.getInvocations();
		TraceOrderList l;
		if(mil.size() > 0) {
			l = new TraceOrderList();
			l._posInList = 0;
			l._listSize = mil.size();
			l._objects = mil;
			l._isEnter = true;
			l._nextId = getNextLowestFromList(l);
			lists.add(l);
		}

		if(mil.size() > 0) {		
			mil = p.getInvocations();
			l = new TraceOrderList();
			l._posInList = 0;
			l._listSize = mil.size();
			l._objects = mil;
			l._isEnter = false;
			l._nextId = getNextLowestFromList(l);
			lists.add(l);
		}
		
			
		return lists;
		
		
	}
	
	
	// --------------------------------------------
	
	@SuppressWarnings("unused")
	@Deprecated
	private static void outputItems(ITraceElementOutput xe, OutputStream os, List<TraceOrderList> lists) throws IOException {
		
		TraceOrderList lowestList = null;
		long lowestId = Long.MAX_VALUE;
		do {
			lowestId = Long.MAX_VALUE;
			lowestList = null;
			
			// Find the next item in the lists
			for(TraceOrderList l : lists) {
				if(!l._objects.isEmpty() && l._nextId < lowestId && l._nextId != Long.MAX_VALUE ) {
					lowestId = l._nextId;
					lowestList = l;
				}
			}
			
			if(lowestList != null) {
				
				Object o = lowestList._objects.get(lowestList._posInList);
				lowestList._listSize--;
				lowestList._posInList++;
				
				outputObject(xe, os, lowestList, o);
				
				if(lowestList._listSize == 0) {
					lists.remove(lowestList);
					lowestList._nextId = Long.MAX_VALUE;
				} else {				
					lowestList._nextId = getNextLowestFromList(lowestList);
				}
				
			}
			
		} while(lowestList != null && lowestId != Long.MAX_VALUE);
		
	}
	
	/** Class that contains a list of sorted objects. Objects should be sorted in chronological order. _nextId
	 * refers to 'trace order' of the next item in the list. See defn at top of class. */
	static class TraceOrderList {
		@SuppressWarnings("rawtypes")
		List  _objects; // = new ArrayList<Object>();
		long _nextId = 0;
		
		int _posInList = 0;
		int _listSize = -1;
		
		/** Whether or not this particular enter/exit is an enter (false is exit) */
		boolean _isEnter = false; 
	}
		
}


