/**********************************************************************
 * Copyright (c) 2012 IBM Corporation
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: AttachDelegate.java,v 1.9 2008-06-06 00:17:41 jkubasta Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.tptp.platform.jvmti.client.internal.launcher;


import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.eclipse.debug.core.model.IStreamsProxy;
import org.eclipse.hyades.execution.core.IDataProcessor;
import org.eclipse.tptp.platform.execution.client.core.IConsole;
import org.eclipse.tptp.platform.execution.client.core.IProcess;

/**
 * A proxy to the streams provided by the <code>Process</code>. Needed by the console view.
 */
public class IOProxy implements IStreamsProxy, IStreamMonitor, org.eclipse.tptp.platform.execution.client.core.IDataProcessor
{
	/** The listeners of this IO proxy */
	private List<IStreamListener> listeners;

	/** The buffer of this IO Proxy */
	private StringBuffer buffer;

	/** The AC console of this proxy */
	private IConsole console;

	/** BUG 312125 - Store the console encoding */
	private String encoding;
	
	/**
	 * Creates a new proxy.
	 */
	public IOProxy(IProcess process, String consoleEncoding)
	{
		console = process.getConsole();
		console.setDataProcessor(this);
		encoding = consoleEncoding;
		buffer = new StringBuffer();
		listeners = new ArrayList<IStreamListener>();
	}


	/**
	 * @see IStreamsProxy#getErrorStreamMonitor()
	 */
	public IStreamMonitor getErrorStreamMonitor()
	{
		return null;
	}


	/**
	 * @see IStreamsProxy#getOutputStreamMonitor()
	 */
	public IStreamMonitor getOutputStreamMonitor()
	{
		return this;
	}


	/**
	 * @see IStreamsProxy#write()
	 */
	public void write(String input) throws IOException
	{
		console.write(input);
	}


	/**
	 * @see IStreamMonitor#addListener()
	 */
	public void addListener(IStreamListener listener)
	{
		synchronized (listeners)
		{
			listeners.add(listener);
		}
	}


	/**
	 * @see IStreamMonitor#removeListener()
	 */
	public void removeListener(IStreamListener listener)
	{
		synchronized (listeners)
		{
			listeners.remove(listener);
		}
	}


	/**
	 * @see IStreamMonitor#getContents()
	 */
	public String getContents()
	{
		return buffer.toString();
	}


	/**
	 * @see IDataProcessor#incommingData()
	 */
	public void incomingData(byte[] buffer, int length, InetAddress peer)
	{
		// BUG 312125: General solution to bug 228936
		//  Convert the buffer into a string using the encoding specified in the
		//  ILaunch. This should be the "right thing" in 99% of the cases and if
		//  it isn't, the user can explicitly select the character encoding in the
		//  launch config (on the "Common" tab)
		String str = null;
		try {
			// BUG 312125: Create the string using the console encoding specified in the ILaunch
			str = new String(buffer, 0, length, this.encoding);
		} catch (UnsupportedEncodingException e) {
			// If that fails, just use the JVM default encoding. 
			str = new String(buffer, 0, length);
		}
		this.buffer.append(str);
		synchronized (listeners)
		{
			Iterator<IStreamListener> i = listeners.iterator();
			while (i.hasNext())
			{
				i.next().streamAppended(str, this);
			}
		}
	}


	/**
	 * @see IDataProcessor#incommingData()
	 */
	public void incomingData(char[] buffer, int length, InetAddress peer)
	{
		byte[] buf = new String(buffer).getBytes();
		incomingData(buf, buf.length, peer);
	}


	/**
	 * @see IDataProcessor#invalidDataType()
	 */
	public void invalidDataType(byte[] arg0, int arg1, InetAddress arg2)
	{
	}


	/**
	 * @see IDataProcessor#waitingForData()
	 */
	public void waitingForData()
	{
	}
}