/**********************************************************************
 * Copyright (c) 2011 IBM Corporation, Intel Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.tptp.platform.jvmti.client.internal.launcher;

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.hyades.trace.ui.internal.launcher.IProfileLaunchConfigurationConstants;

/**
 * Abstracts a structured representation of a -agent(lib|path): command line. Manipulators are 
 * provided so that a TI command line argument can be built up piece by piece.
 * 
 * A list of auxiliary command line arguments are also accommodated since some JVMTI invocations 
 * require additional JVM configuration options (e.g. -Xbootclasspath, etc).
 * 
 * @author mike
 *
 */
public class TICommandline {

	public static final String AGENT_CGPROF = "CGProf";//$NON-NLS-1$
	public static final String AGENT_HEAPPROF = "HeapProf";//$NON-NLS-1$
	public static final String AGENT_THREADPROF = "ThreadProf";//$NON-NLS-1$
	public static final String AGENT_PROBEKIT = "ProbekitAgent";//$NON-NLS-1$
	
	public static final String MODE_CONTROLLED = "controlled";//$NON-NLS-1$
	public static final String MODE_ENABLED = "enabled";//$NON-NLS-1$
	public static final String MODE_STANDALONE = "standalone";//$NON-NLS-1$
	
	public static final String JPI_AGENT = "JPIAgent";//$NON-NLS-1$
	public static final String DEFAULT_AGENT = AGENT_CGPROF;//$NON-NLS-1$

	public static final String DEFAULT_BOOTLOADER = "JPIBootLoader";//$NON-NLS-1$
	
	private String agentPath;
	private String bootLoader;
	
	private String agent;
	
	private Map<String,String> jpiOptions;
	private Map<String,String> agentOptions;
	
	private List<String> auxiliaryArguments;

	/**
	 * Construct a TI command line. 
	 * 
	 * Functionally equivalent to:
	 * 
	 * 	new TICommand( DEFAULT_JPI, "server", "controlled" );
	 */
	public TICommandline() {
		this( "controlled" );//$NON-NLS-1$//$NON-NLS-2$
	}
	
	/**
	 * Construct a TI command line. JPIAgent server mode is configured with the specified `mode`. 
	 * The ...opts array is interpreted pair-wise, as a list of option-values applied to JPIAgent.
	 * 
	 * Agent defaults to DEFAULT_AGENT with no options.
	 * 
	 * @param mode - JPIAgent mode: 'enabled', 'standalone', or 'controlled' are accepted
	 * @param opts - pairwise list of option-values
	 */
	public TICommandline( String mode, String...opts ) {
	
		this.agent = DEFAULT_AGENT;
		this.agentPath = null;
		this.bootLoader = DEFAULT_BOOTLOADER;
	
		this.jpiOptions = new LinkedHashMap<String, String>();
		this.agentOptions = new LinkedHashMap<String, String>();
		this.auxiliaryArguments = new LinkedList<String>();

		this.jpiOptions.put( "server", mode );//$NON-NLS-1$
		this.jpiOptions( opts );
		
	}
	
	/**
	 * Specify the path to the agent bootloader library. After specifying this, the generated
	 * command line will use -agentpath: instead of -agentlib:
	 *  
	 * @param path - Path to JVMTI libs.
	 * @return this
	 */
	public TICommandline agentPath( String path ) {
	
		this.agentPath = path.endsWith(File.separator) ? path.substring(0, path.length()-1) : path;	
		return this;
		
	}
	
	/**
	 * @return The current value of agentpath; `null' indicates no path has been given
	 */
	public String getAgentPath() {
		return this.agentPath;
	}
	
	/**
	 * Set the JVMTI bootloader. There is currently no reason to set this to anything other than
	 * the default (DEFAULT_BOOTLOADER). There may never be a reason to change, provided here for
	 * completeness.
	 *  
	 * @param bootLoader
	 * @return this
	 */
	public TICommandline bootloader( String bootLoader ) {
		
		this.bootLoader = bootLoader;
		return this;
		
	}
	
	/**
	 * Set the name of the JPI library and associated options. Currently, `jpi' should never be
	 * anything other than "JPIAgent" (DEFAULT_JPI).
	 * 
	 * The current set of jpiOptions and agentOptions are cleared. The given pair-wise list of
	 * options replace the previous set of `jpiOptions'. Agent options are simply discarded. 
	 * 
	 * @param mode - Launch mode for JPI agent library; one of: "controlled", "standalone", "enabled"
	 * @param opts - Pairwise list of option-values to be applied as arguments to the JPI library
	 * @return this
	 */
	public TICommandline jpi( String mode, String... opts ) {
		
		this.jpiOptions.clear();
		this.agentOptions.clear();
		
		this.jpiOptions.put("server", mode);//$NON-NLS-1$
		for( int i=0; i<opts.length/2; i++ )
			this.jpiOptions.put( opts[2*i], opts[2*i+1] );
		
		return this;
		
	}
	
	/**
	 * Set the profiling agent and associated options.
	 * 
	 * The current list of agent options is discarded and replace with the option-value pairs
	 * specified in `opts'.
	 * 
	 * @param agent - Profiling agent
	 * @param opts - Pairwise list of option-values
	 * @return this
	 */
	public TICommandline agent( String agent, String... opts ) {
		
		this.agent = agent;
		this.agentOptions.clear();
		
		return agentOptions( opts );

	}
	
	/**
	 * Specify an option to the JPI library. Overwrites any previously specified option of the same 
	 * name.
	 * 
	 * @param option Option name
	 * @param value Option value
	 * @return this
	 */
	public TICommandline jpiOption( String option, String value ) {
		
		this.jpiOptions.put( option, value );
		return this;
		
	}

	/**
	 * Specify multiple options to the JPI library. Overwrites any existing entries with same name.
	 * 
	 * @param options A Map<String,String> of options to associate w/ JPI lib
	 * @return this
	 */
	public TICommandline jpiOptions( Map<String,String> options ) {
		
		this.jpiOptions.putAll(options);
		return this;
		
	}
	
	/**
	 * Specify multiple options to the JPI library. Overwrites existing entries of the same name.
	 * 
	 * @param options Pairwise list of option-values.
	 * @return this
	 */
	public TICommandline jpiOptions( String... options ) {
		
		for( int i=0; i<options.length/2; i++ )
			this.jpiOptions.put( options[2*i], options[2*i+1] );		
		return this;
		
	}
	
	/**
	 * Specify an option to the agent library. Overwrites any existing entry of the same name.
	 * 
	 * @param option Option to specify
	 * @param value Value of specified option
	 * @return this
	 */
	public TICommandline agentOption( String option, String value ) {
		
		this.agentOptions.put( option, value );
		return this;
		
	}
	
	/**
	 * Specify multiple options to the agent library. Overwrites any existing entries with same name.
	 * 
	 * @param options A Map<String,String> of options to associate w/ agent library
	 * @return this
	 */
	public TICommandline agentOptions( Map<String,String> options ) {
		
		this.agentOptions.putAll(options);
		return this;
		
	}

	/**
	 * Specify multiple options to the agent library. Overwrites existing entries of the same name.
	 * 
	 * @param options Pairwise list of option-values.
	 * @return this
	 */
	public TICommandline agentOptions( String... options ) {
		
		for( int i=0; i<options.length/2; i++ )
			this.agentOptions.put( options[2*i], options[2*i+1] );		
		return this;
		
	}
	
	/**
	 * Add an auxiliary argument to the command line. The argument should be unquoted, quoting is
	 * performed automatically when the full command-line is requested.
	 * 
	 * @param arg 
	 * @return this
	 */
	public TICommandline auxiliaryArgument( String arg ) {
	
		this.auxiliaryArguments.add(arg);
		return this;
		
	}
	
	/**
	 * Adds the collection of auxiliary arguments to the generated command line. The arguments
	 * should be unquoted; quoting is performed automatically when the full command line is 
	 * requested.
	 * 
	 * @param args List of arguments to add to the command line
	 * @return
	 */
	public TICommandline auxiliaryArguments( List<String> args ) {
		
		this.auxiliaryArguments.addAll(args);
		return this;
		
	}
	
	private static void appendOptions( StringBuilder builder, Map<String,String> opts ) {
		
		if( !opts.isEmpty() ) {
			builder.append(':');

			boolean firstEntry = true;
			for( Map.Entry<String, String> opt : opts.entrySet() ) {
				
				if( !firstEntry )
					builder.append(',');
				builder.append( opt.getKey() );
				builder.append( '=' );
				builder.append( opt.getValue() );
				
				firstEntry = false;
			}
		}
		
	}
	
	/**
	 * Build the full command line specified by this object. The command line is generated as follows:
	 * 
	 * 1. Auxiliary arguments are quoted and concatenated together, each separated by a space
	 * 2. The TI command line is generated according to the schema:
	 * 
	 * 	-agent(lib|path):[<agentPath>/]=JPIAgent:jpiOption1=jpiValue1[,jpiOption2=jpiValue2...];<agent>[:agentOption1=agentValue1[,agentOption2=agentValue2...]
	 * 
	 * 3. The TI command line is quoated and append to the auxiliary argument string.
	 * 
	 * The resulting string is properly quoted and can be used to launch a process through the AC
	 * with no further processing. 
	 * 
	 * @return String which can be used to launch a JVM with the specified JVMTI agent options.
	 */
	public String getCommandline() {
		
		// getAux.. returns a trailing space, so this works :)
		StringBuilder cmdline = new StringBuilder();
		for( String arg : this.auxiliaryArguments ) {
			
			cmdline.append('"');
			cmdline.append(arg);
			cmdline.append('"');
			cmdline.append(' ');
			
		}
		cmdline.append( getTIArgument() );
		return cmdline.toString();
		
	}

	/**
	 * Retrieve the command as a java.util.List<String>.
	 * 
	 * @return List<String> of VM arguments.
	 */
	public List<String> getVMArguments() {
		
		ArrayList<String> args = new ArrayList<String>( this.auxiliaryArguments.size() + 1 );
		args.addAll( this.auxiliaryArguments );
		args.add( getTIArgument() );
		
		return args;
		
	}
	
	/**
	 * Retrieve the auxiliary JVM arguments associated with this command line. The JVMTI agent
	 * option is not included.
	 * 
	 * @return
	 */
	public List<String> getAuxiliaryArguments() {
		
		return new ArrayList<String>( this.auxiliaryArguments );
		
	}
	
	/**
	 * Retrieve the JVMTI -agent(lib|path) option as currently specified. The argument is
	 * properly quoted.
	 * 
	 * @return
	 */
	public String getTIArgument() {
		
		StringBuilder arg = new StringBuilder();
		
		arg.append('"');
		if( null != agentPath ) {
			
			arg.append( "-agentpath:" );//$NON-NLS-1$
			arg.append( this.agentPath );
			arg.append( File.separatorChar );
			arg.append( IProfileLaunchConfigurationConstants.LIB_PREFIX );
			arg.append( this.bootLoader );
			arg.append( IProfileLaunchConfigurationConstants.LIB_SUFFIX );
			
		} else {
			
			arg.append( "-agentlib:" );//$NON-NLS-1$
			arg.append( this.bootLoader );
			
		}
		
		arg.append('=');
		arg.append(JPI_AGENT);
		appendOptions( arg, this.jpiOptions );
		
		arg.append( ';' );
		arg.append( this.agent );
		appendOptions( arg, this.agentOptions );
		
		arg.append('"');
		
		return arg.toString();
		
	}
	
}
